package org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Response;

import org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.IRM;
import org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Prepared;
import org.eclipse.tm.tcf.protocol.IChannel;
import org.eclipse.tm.tcf.protocol.IToken;
import org.eclipse.tm.tcf.services.IFileSystem;
import org.eclipse.tm.tcf.services.IFileSystem.IFileHandle;
import org.eclipse.tm.tcf.services.IFileSystem.FileSystemException;

public class RMFileSystem implements IRM{
	
	public static int
		PLAN_DELETE_FILE = 1,
		PLAN_DELETE_DIR = 2,
		PLAN_TRANSIENT_DROP_PING_PACKETS = 3,         // This will cause the kernel to simply ignore all ping messages (also known as ICMP type 0 messages)
	    											  // # /bin/echo "1" > /proc/sys/net/ipv4/icmp_echo_ignore_all
	    
		PLAN_TRANSIENT_DEACTIVATE_IP_FORWARDING = 4,  // Make sure that IP forwarding is turned off
													  // # /bin/echo "0" > /proc/sys/net/ipv4/ip_forward
		
	    PLAN_TRANSIENT_IGNORE_BROADCAST_PINGS = 5,    // This disables response to ICMP broadcasts and will prevent Smurf attacks. The Smurf attack works by 
													  // sending an ICMP type 0 (ping) message to the broadcast address of a network. Typically the attacker 
													  // will use a spoofed source address. All the computers on the network will respond to the ping message 
													  // and thereby flood the host at the spoofed source address
													  // # /bin/echo "1" > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts
	    
		PLAN_TRANSIENT_DISABLE_SOURCE_ROUTED_PACKETS = 6, // Do not accept source routed packets. Attackers can use source routing to generate traffic pretending
												 		   // to originate from inside your network, but that is actually routed back along the path from which it 
												 		   // came, so attackers can compromise your network. Source routing is rarely used for legitimate purposes
												           // , so it is safe to disable it
												           // # /bin/echo "0" > /proc/sys/net/ipv4/conf/all/accept_source_route
		
		PLAN_TRANSIENT_DISABLE_REDIRECT_ACCEPTANCE = 7,   // Do not accept ICMP redirect packets. ICMP redirects can be used to alter your routing tables, possibly 
												           // to a malicious end.
												           // # /bin/echo "0" > /proc/sys/net/ipv4/conf/all/accept_redirects
	
	    PLAN_TRANSIENT_PROTECT_AGAINST_BAD_ERROR_MESSAGES = 8, // Enable protection against bogus error message responses
													            // # /bin/echo "1" > /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses
	
		PLAN_TRANSIENT_ENABLE_REVERSE_PATH_FILTERING = 9,    // Turn on reverse path filtering. This helps make sure that packets use legitimate source addresses
													          // by automatically rejecting incoming packets if the routing table entry for their source address
													          // does not match the network interface they are arriving on. This has security advantages because 
													          // it prevents IP spoofing. We need to reenable it for each net/ipv4/conf/* otherwise source validation 
													          // isn't fully functional
													          // # for i in /proc/sys/net/ipv4/conf/*; do
    												          //        /bin/echo "1" > $i/rp_filter
    												          // done
		
	    PLAN_TRANSIENT_LOG_ALL_S_SR_RP = 10,                  // Log spoofed packets, source routed packets and redirect packets
													          // # /bin/echo "1" > /proc/sys/net/ipv4/conf/all/log_martians
	    
	    PLAN_MAX_LOGINS = 11,         // Max number of logins for this user: (/etc/security/limits.conf)
		PLAN_MAX_FILE_LOCKS = 12,     // Max number of file locks the user can hold: (/etc/security/limits.conf)
									 // It has a format to add a rule: <domain>  <type>  <item>  <value>
		PLAN_PERMANENT_DROP_PING_PACKETS = 13,
	    PLAN_PERMANENT_DEACTIVATE_IP_FORWARDING = 14,		  // add them to /etc/sysctl.conf, which is automatically sourced by the /etc/init.d/bootmisc init script.
	    													  //	net.ipv4.ip_forward = 0
	    PLAN_PERMANENT_IGNORE_BROADCAST_PINGS = 15,
	    PLAN_PERMANENT_DISABLE_SOURCE_ROUTED_PACKETS = 16,
	    PLAN_PERMANENT_DISABLE_REDIRECT_ACCEPTANCE = 17,
	    PLAN_PERMANENT_PROTECT_AGAINST_BAD_ERROR_MESSAGES = 18,
	    PLAN_PERMANENT_ENABLE_REVERSE_PATH_FILTERING = 19,
	    PLAN_PERMANENT_LOG_ALL_S_SR_RP = 20;

	    
	    
		
    private final Prepared test_suite;
    private final int channel_id;
    
    private final int plan_id;


    private final IFileSystem files;
    private String root;
    private String tmp_path;
    private String file_name;
    private IFileHandle filehandle;

    public RMFileSystem(Prepared test_suite, IChannel channel, int channel_id, int plan_id) {
        this.test_suite = test_suite;
        this.channel_id = channel_id;
        this.plan_id = plan_id;

        files = channel.getRemoteService(IFileSystem.class);
    }

    public void start() {
        if (files == null) {
            test_suite.done(this, null);
        }
        else {
        	switch (plan_id) {
	          case 1:
				remove();
				break;
	          case 2:
	        	  remove_dir();
	        	  break;
	          case 3:
	        	  write_zero_one("/proc/sys/net/ipv4/icmp_echo_ignore_all","1");
	        	  break;
	          case 4:
	        	  write_zero_one("/proc/sys/net/ipv4/ip_forward","0");
	        	  break;
	          case 5:
	        	  write_zero_one("/proc/sys/net/ipv4/icmp_echo_ignore_broadcasts","1");
	        	  break;
	          case 6:
	        	  write_zero_one("/proc/sys/net/ipv4/conf/all/accept_source_route","0");
	        	  break;
	          case 7:
	        	  write_zero_one("/proc/sys/net/ipv4/conf/all/accept_redirects","0");
	        	  break;
	          case 8:
	        	  write_zero_one("/proc/sys/net/ipv4/icmp_ignore_bogus_error_responses","1");
	        	  break;
	          case 9:
	        	  //write_zero_one("/proc/sys/net/ipv4/ip_forward","0");
	        	  break;
	          case 10:
	        	  write_zero_one("/proc/sys/net/ipv4/conf/all/log_martians","1");
	        	  break;
	          case 11:
	          case 12:
        	}
        }
    }

    public void remove() {
        final String path = "/home/alireza/Desktop/ali.txt";
        files.remove(path, null);
		System.out.println("remove file Plan was run");

	}
    
    public void remove_dir() {
        final String path = "/home/alireza/Desktop/salam";
        files.rmdir(path, null);
		System.out.println("remove directory Plan was run");

	}
    
    public void write_zero_one(String path, final String strContent) {
    	//final String path = "/home/alireza/Desktop/salam.txt";
    	files.open(path, IFileSystem.TCF_O_CREAT | IFileSystem.TCF_O_TRUNC | IFileSystem.TCF_O_WRITE, null, new IFileSystem.DoneOpen() {
            public void doneOpen(IToken token, FileSystemException error, IFileHandle handle) {
            	if (error != null) {
                    exit(error);
                }
                else {
                    //String strContent = "1";
                    filehandle = handle;
                    files.write(handle, 0, strContent.getBytes(), 0,1 , new IFileSystem.DoneWrite() {

							@Override
							public void doneWrite(IToken token,
									FileSystemException error) {
								// TODO Auto-generated method stub
								if (error != null) {
	                                exit(error);
	                            }
	                            else {
	                            	files.close(filehandle, new IFileSystem.DoneClose() {
									
										@Override
										public void doneClose(IToken token, FileSystemException error) {
											// TODO Auto-generated method stub
											System.out.println("transient_deactivate_ip_forwarding Plan was run");

										}
									});
	                            }
							}
                        	
                        });
                  }
            }
        });
	}
    

    private void exit(Throwable x) {
        if (!test_suite.isActive(this)) return;
        test_suite.done(this, x);
    }
}