package org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Response;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.tm.tcf.protocol.IChannel;
import org.eclipse.tm.tcf.protocol.IToken;
import org.eclipse.tm.tcf.services.IFileSystem;
import org.eclipse.tm.tcf.services.IProcesses;
import org.eclipse.tm.tcf.services.IFileSystem.FileSystemException;
import org.eclipse.tm.tcf.services.IFileSystem.IFileHandle;
import org.eclipse.tm.tcf.services.IProcesses.ProcessContext;

/**
 * This Reactive Measure will cause the kernel to ignore all ping messages
 * To make the change PERMANENT add the following line to /etc/sysctl.conf:
 * net.ipv4.icmp_echo_ignore_all=1
 */

public class RMPerDropPingPackets implements IReactiveMeasures {
	static public String Name = "RM_PERMANENT_DROP_PING_PACKETS";
	static public int Id = 26;
	static public double Risk_threshold = 0.6;
    
    private final IProcesses processes;

    public RMPerDropPingPackets(IChannel channel) {
            
    	processes = channel.getRemoteService(IProcesses.class);

    }
    
    private String[] toArgsArray(String file, String cmd) {
        // Create arguments list from a command line.
        int i = 0;
        int l = cmd.length();
        List<String> arr = new ArrayList<String>();
        arr.add(file);
        for (;;) {
            while (i < l && cmd.charAt(i) == ' ') i++;
            if (i >= l) break;
            String s = null;
            if (cmd.charAt(i) == '"') {
                i++;
                StringBuffer bf = new StringBuffer();
                while (i < l) {
                    char ch = cmd.charAt(i++);
                    if (ch == '"') break;
                    if (ch == '\\' && i < l) ch = cmd.charAt(i++);
                    bf.append(ch);
                }
                s = bf.toString();
            }
            else {
                int i0 = i;
                while (i < l && cmd.charAt(i) != ' ') i++;
                s = cmd.substring(i0, i);
            }
            arr.add(s);
        }
        return arr.toArray(new String[arr.size()]);
	}

	@Override
	public void start() {
	    // First step: delete previous pattern
		removePattern();
	    // Second step: add new pattern
		addPattern();
		System.out.println(Name + " with " + Risk_threshold + " threshold was run");
		
	}
	
	private void removePattern() {
		String dir = "/home/";
		String file = "/bin/sed";
	    String args = "-i \"/[\t| ]*net.ipv4.icmp_echo_ignore_all/d\" /etc/sysctl.conf";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
					System.out.println(error.toString());
				}		
		});
	}
	
	private void addPattern() {
		String dir = "/home/";
		String file = "/bin/sed";
	    String args = "-i \"1i\\net.ipv4.icmp_echo_ignore_all=1\" /etc/sysctl.conf";
	    
	    processes.start(dir, file, toArgsArray(file, args), null , false, new IProcesses.DoneStart() {      	  
				public void doneStart(IToken token, Exception error, ProcessContext process) {
					System.out.println(error.toString());
				}		
		});
	}
	
}
