package org.eclipse.linuxtools.lttng.SYSHEALTHPRO.Prevention.Response;

import org.eclipse.tm.tcf.protocol.IChannel;
import org.eclipse.tm.tcf.protocol.IToken;
import org.eclipse.tm.tcf.services.IFileSystem;
import org.eclipse.tm.tcf.services.IFileSystem.FileSystemException;
import org.eclipse.tm.tcf.services.IFileSystem.IFileHandle;

/**
 * This Reactive Measure will disable response to ICMP broadcasts and prevent Smurf attacks. 
 * The Smurf attack works by sending an ICMP type 0 (ping) message to the broadcast address of a network. 
 * Typically the attacker will use a spoofed source address. All the computers on the network will respond 
 * to the ping message and thereby flood the host at the spoofed source address
 * /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts's value must be 1
 */

public class RMTranIgnoreBroadcastPings implements IReactiveMeasures {
	static public String Name = "RM_TRANSIENT_IGNORE_BROADCAST_PINGS";
	static public int Id = 18;
	static public double Risk_threshold = 0.6;
    
    private final IFileSystem files;
    private IFileHandle filehandle;



    public RMTranIgnoreBroadcastPings(IChannel channel) {
            
    	files = channel.getRemoteService(IFileSystem.class);

    }
    
	@Override
	public void start() {
		final String path = "/proc/sys/net/ipv4/icmp_echo_ignore_broadcasts";
		final String strContent = "1";
		
		files.open(path, IFileSystem.TCF_O_CREAT | IFileSystem.TCF_O_TRUNC | IFileSystem.TCF_O_WRITE, null, new IFileSystem.DoneOpen() {
            public void doneOpen(IToken token, FileSystemException error, IFileHandle handle) {
            	if (error != null) {
            		System.out.println(error);
                }
                else {
                    filehandle = handle;
                    files.write(handle, 0, strContent.getBytes(), 0,1 , new IFileSystem.DoneWrite() {

							@Override
							public void doneWrite(IToken token,
									FileSystemException error) {
								if (error != null) {
									System.out.println(error);
	                            }
	                            else {
	                            	files.close(filehandle, new IFileSystem.DoneClose() {
										@Override
										public void doneClose(IToken token, FileSystemException error) {
											System.out.println(Name + " with " + Risk_threshold + " threshold was run");
										}
									});
	                            }
							}
                        	
                        });
                  }
            }
        });
		
	}
	
}
