package org.eclipse.linuxtools.lttng.SYSPRO.Prediction;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DecimalFormat;

import net.sourceforge.jFuzzyLogic.FIS;

import org.eclipse.core.runtime.Platform;
import org.eclipse.linuxtools.lttng.SYSPRO.ui.Application;
import org.eclipse.linuxtools.lttng.SYSPRO.ui.Connect;
import org.eclipse.ui.internal.util.BundleUtility;
import org.osgi.framework.Bundle;



public class Prediction {

	Connect c = new Connect();
	ResultSet rs = null;
	int statesNb = 0;
	int observationsNb = 0;
	int Flag_first = 0;

	double[] alpha;
	double[] gama;
	double[][] transitionProbability ;
	double[][] emissionProbability;
	double[] startProbability;

	public Prediction () {
		// Initialize 
		int i, j, ObsIndex, sendingIndex;

		statesNb = ReadstatesNb();
		observationsNb = ReadObservationsNb();

		alpha = new double[statesNb];
		gama = new double[statesNb];

		transitionProbability = new double[statesNb][statesNb];
		emissionProbability = new double[statesNb][observationsNb];
		startProbability = new double[statesNb];

		startProbability [0] = 1.0; 
		startProbability [1] = 0.0;
		startProbability [2] = 0.0;
		startProbability [3] = 0.0;

		for (i= 0; i < statesNb; i++) {	
			for (j= 0; j < statesNb; j++) {
				transitionProbability[i][j] = ReadTransitionValue(i+1,j+1);
			}
		}

		for (i= 0; i < statesNb; i++) {	
			for (j= 0; j < observationsNb; j++) {
				emissionProbability[i][j] = ReadEmissionValue(i+1,j+1);
			}
		}

	}

	public double[] run(int sendingIndex) {
		int i, j;
		double sum;
		double sumAlpha;
		// Start Analysis
		System.out.println("\tStarting of Hidden Markov Model analysis...\n");

		if (Flag_first == 0) {
			
			sumAlpha = 0;
			for (i= 0; i < statesNb; i++) {	
				alpha[i]= startProbability[i] * emissionProbability[i][sendingIndex-1];
				sumAlpha += alpha[i];
			}
			for (i= 0; i < statesNb; i++) {	
				gama[i]= alpha[i]/sumAlpha;
				System.out.println("\t" + gama[i]);
			}
			Flag_first = 1;
			
		} else {

			sumAlpha = 0;
			sum = 0;
			for (i= 0; i < statesNb; i++) {
				for (j= 0; j < statesNb; j++) {
					sum +=gama[j] * transitionProbability[j][i];
				}
				alpha[i]= emissionProbability[i][sendingIndex-1] * sum;
				sumAlpha += alpha[i];
				sum = 0;
			}

			for (i= 0; i < statesNb; i++) {	
				gama[i]= alpha[i]/sumAlpha;
				System.out.println("\t" + gama[i]);
			}
			
		}
		//if (gama[3] > 0.5)
			System.out.println("Compromize probability: " + gama[3]);
		return gama;

	}

	private int ReadstatesNb() {
		ResultSet rs = null;
		int Rowcount = -1;

		try {
			rs = c.Query("SELECT COUNT(*) FROM HMMStates");
			rs.next();
			Rowcount = rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return Rowcount;
	}


	private int ReadObservationsNb() {
		ResultSet rs = null;
		int Rowcount = -1;

		try {
			rs = c.Query("SELECT COUNT(*) FROM HMMObservations");
			rs.next();
			Rowcount = rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return Rowcount;
	}

	private double ReadTransitionValue(int StateId1, int StateId2) {
		double value = 0.0;

		ResultSet rs = null;
		try {
			rs = c.Query("select * from TransitionProbability where StateId1=" + StateId1 + " and StateId2=" + StateId2);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		try {
			while (rs.next ())
			{
				value =  Double.parseDouble(rs.getString ("Value"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return value;
	}

	private double ReadEmissionValue(int StateId, int ObservationeId) {
		double value = 0.0;

		ResultSet rs = null;
		try {
			rs = c.Query("select * from EmissionProbability where StateId=" + StateId + " and ObservationId=" + ObservationeId);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		try {
			while (rs.next ())
			{
				value =  Double.parseDouble(rs.getString ("Value"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return value;
	}

}
