/*
 * @(#)StateImpl.java	1.10 06/08/06
 *
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm ;

import java.util.Map ;
import java.util.HashMap ;
import java.util.Set ;
import java.util.HashSet ;
import java.util.Iterator;


/** Base class for all states in a StateEngine.  This must be used
 * as the base class for all states in transitions added to a StateEngine.
 */
public class StateImpl extends NameBase implements State {
	private Action defaultAction ;
	private State defaultNextState ;
	private Map inputToGuardedActions ;
	
	HashMap nestedHashMap = new HashMap(); 
	
	public StateImpl( String name ) 
	{
		super( name ) ; 
		defaultAction = null ;
		inputToGuardedActions = new HashMap() ;
	}

	public void preAction( FSM fsm ) 
	{
	}

	public void postAction( FSM fsm )
	{
	}

	// Methods for use only by StateEngineImpl.

	public State getDefaultNextState()
	{
		return defaultNextState ;
	}

	public void setDefaultNextState( State defaultNextState )
	{
		this.defaultNextState = defaultNextState ;
	}

	public Action getDefaultAction()
	{
		return defaultAction ;
	}

	public void setDefaultAction( Action defaultAction )
	{
		this.defaultAction = defaultAction ;
	}

	public void addGuardedAction( Input in, GuardedAction ga)
	{
		Set gas = (Set)inputToGuardedActions.get( in ) ;
		if (gas == null) {
			gas = new HashSet() ;
			inputToGuardedActions.put( in, gas ) ;
		}

		gas.add( ga ) ;
	}
	
	public void increaseStateInput( State oldState, Input in, State nextState)
	{
		HashMap tempMap = new HashMap();
		Input incheck;

    	tempMap.put(oldState, in);

	    if(!nestedHashMap.containsKey(tempMap)) {
	    		nestedHashMap.put(tempMap, nextState);
	    }
	    
	    Set set = nestedHashMap.entrySet(); 
		Iterator i = set.iterator(); 
		
		while(i.hasNext()) { 
				Map.Entry me = (Map.Entry)i.next();
				//System.out.print(me.getKey() + ": "); 
				//System.out.println(me.getValue()); 
		}
	}

	public Set getGuardedActions( Input in )
	{
		return (Set)inputToGuardedActions.get( in ) ;
	}
	
	public boolean decreaseStateInput( State oldState, Input in, State nextState)
	{
		boolean permission= true;
		HashMap tempMap = new HashMap();
		State statecheck;

		tempMap.put(oldState, in);

	    if(nestedHashMap.containsKey(tempMap)) {
	    		nestedHashMap.remove(tempMap);
	    		System.out.println(oldState + " " + in + " " + nextState); 
	    }
	    
	    Set set = nestedHashMap.entrySet(); 
		Iterator i = set.iterator(); 
		
		while(i.hasNext()) { 
			Map.Entry me = (Map.Entry)i.next();

			tempMap= (HashMap) me.getKey();

		    if(tempMap.containsKey(oldState) && me.getValue()==nextState) {
		    	permission= false;
		    }
			//System.out.print(me.getKey() + ": "); 
			//System.out.println(me.getValue()); 
		}
		return permission;
	}
}