/*
 * @(#)FSMTest.java	1.12 06/08/06
 *
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package org.eclipse.linuxtools.lttng.SYSPRO.AttackGraph ;

import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.Input ;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.Action ;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.ActionBase;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.Guard ;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.GuardBase;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.StateEngine ;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.StateImpl ;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.StateEngineFactory ;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.FSM ;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.FSMImpl;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.State;
import org.eclipse.linuxtools.lttng.SYSPRO.dynamicfsm.InputImpl;

public class AttackGraph extends FSMImpl {
	private final Thread[] etherealizer ;   // The actual etherealize operation 
	// for this entry.  It is 
	// represented as a Thread because
	// the POA.deactivate_object never 
	// waits for the completion.
	private final int[] counter ;	    // single element holder for counter 
	//private SyntheticEvent syntheticevent;
	
	
	public static final State START0 = new StateImpl( "START0" );
	public static final State APACHE_SHELL = new StateImpl( "APACHE_SHELL" );
	public static final State NCAT = new StateImpl( "NCAT" );
	public static final State WGET = new StateImpl( "WGET" );
	public static final State COMPILE = new StateImpl( "COMPILE" );
	public static final State EXPLOIT   = new StateImpl( "EXPLOIT" );
	public static final State EXPLOIT_SHELL   = new StateImpl( "EXPLOIT SHELL" );
	public static final State COMPROMIZE_ROOT    = new StateImpl( "ROOT PERMISSION BY ATTACKER" )

	{

		public void preAction( FSM fsm ) {
			AttackGraph entry = (AttackGraph)fsm;
		}
		public void postAction( FSM fsm ) {
			AttackGraph entry = (AttackGraph)fsm;
		}
	};

	static final Input SERVICE_APACHE  	= new InputImpl( "apache is running" );
	static final Input EXECUTE_APACHE_SHELL   	= new InputImpl( "apache executes shell" );
	static final Input EXECUTE_SHELL_NCAT	    = new InputImpl( "shell executes ncat" ) ;
	static final Input CONNECT_NCAT_REMOTEHOST	    = new InputImpl( "ncat connects to a remote host" );
	static final Input EXECUTE_SHELL_WGET	    = new InputImpl( "shell executes wget" );
	static final Input EXECUTE_NCAT_WGET	    = new InputImpl( "ncat executes wget" );
	static final Input EXECUTE_SHELL_CC	    = new InputImpl( "shell executes compile" );
	static final Input EXECUTE_NCAT_CC	    = new InputImpl( "ncat executes compile" );
	static final Input EXECUTE_SHELL_EXPLOIT    	= new InputImpl( "shell executes exploit" );
	static final Input EXECUTE_NCAT_EXPLOIT    	= new InputImpl( "ncat executes exploit" );
	static final Input EXECUTE_EXPLOIT_SHELL	    = new InputImpl( "exploit program creates shell" );
	static final Input EXECUTE_SHELL_ADDUSER    = new InputImpl( "shell creates a user" );
	static final Input BACKNCAT    = new InputImpl( "back to ncat state" );
	static final Input BACKSHELL    = new InputImpl( "back to shell state" );


	private static Action sendAction = new ActionBase( "sendincrement" ) {
		public void doIt( FSM fsm, Input in ) {
			AttackGraph entry = (AttackGraph)fsm ;
			//entry.sendNo ++;
			//entry.readBytes= 
		}
	} ;
	private static Action rcvAction = new ActionBase( "receiveincrement" ) {
		public void doIt( FSM fsm, Input in ) {
			AttackGraph entry = (AttackGraph)fsm ;
			//entry.rcvNo ++;
		}
	} ;

	private static class CounterGuard extends GuardBase {
		private int value ;

		public CounterGuard( int value ) 
		{
			super( "counter>" + value ) ;
			this.value = value ;
		}
		public Guard.Result evaluate( FSM fsm, Input in ) 
		{
			AttackGraph entry = (AttackGraph)fsm ;
			return Guard.Result.convert( entry.counter[0] > value ) ;
		}
	} ;

	private static GuardBase greaterZeroGuard = new CounterGuard( 0 ) ;
	private static Guard zeroGuard = new Guard.Complement( greaterZeroGuard ) ;
	private static GuardBase greaterOneGuard = new CounterGuard( 1 ) ;
	private static Guard oneGuard = new Guard.Complement( greaterOneGuard ) ;

	private static StateEngine engine ;

	static {
		engine = StateEngineFactory.create() ;

		engine.add( START0, SERVICE_APACHE,						null,    APACHE_SHELL);
		engine.add( START0, EXECUTE_APACHE_SHELL,				null,    APACHE_SHELL);
		
		engine.add( APACHE_SHELL, EXECUTE_SHELL_NCAT,			null,    NCAT);
		engine.add( APACHE_SHELL, CONNECT_NCAT_REMOTEHOST,		null,    NCAT);
		engine.add( APACHE_SHELL, EXECUTE_SHELL_WGET,			null,    WGET);

		engine.add( NCAT, EXECUTE_NCAT_WGET,					null,    WGET);
		engine.add( WGET, BACKNCAT,								null,    NCAT);
		engine.add( WGET, BACKSHELL,							null,    APACHE_SHELL);

		engine.add( NCAT, EXECUTE_NCAT_CC,						null,    COMPILE);
		engine.add( COMPILE, BACKNCAT,							null,    NCAT);
		engine.add( COMPILE, BACKSHELL,							null,    APACHE_SHELL);

		engine.add( APACHE_SHELL, EXECUTE_SHELL_EXPLOIT,		null,    EXPLOIT);
		engine.add( EXPLOIT, BACKSHELL,							null,    APACHE_SHELL);

		engine.add( NCAT, EXECUTE_NCAT_EXPLOIT,					null,    EXPLOIT);
		engine.add( EXPLOIT, BACKSHELL,							null,    NCAT);
		engine.add( EXPLOIT, EXECUTE_EXPLOIT_SHELL,				null,    EXPLOIT_SHELL);
		
		engine.add( EXPLOIT_SHELL, EXECUTE_SHELL_ADDUSER,		null,    COMPROMIZE_ROOT);

		
		//engine.add( WGET, EXECUTE_EXPLOIT_SHELL,				null,    EXPLOIT);
		//engine.add( WGET, EXECUTE_SHELL_ADDUSER,				null,    COMPROMIZE_ROOT);


		//engine.add( COMPILE, EXECUTE_SHELL_EXPLOIT,				null,    EXPLOIT);
		//engine.add( COMPILE, EXECUTE_EXPLOIT_SHELL,				null,    EXPLOIT);
		//engine.add( COMPILE, EXECUTE_SHELL_ADDUSER,				null,    COMPROMIZE_ROOT);

		
		//engine.add( EXPLOIT, EXECUTE_SHELL_ADDUSER,				null,    COMPROMIZE_ROOT);

		
		engine.setDefault(START0);
		engine.done();
	}

	public AttackGraph()
	{
		super( engine, START0, false) ;
		etherealizer = new Thread[1] ;
		etherealizer[0] = null ;
		counter = new int[1] ;
		counter[0] = 0 ;
	}

	// Methods that drive the FSM: the real interface to this class
	// Most just call the doIt method, but startEtherealize needs
	// the etherealizer.
	public void startAttackGraph()
	{
		doIt( SERVICE_APACHE ) ; 
	}
	public void serviceApache() { doIt( SERVICE_APACHE ); }
	public void executeApacheShell() { doIt( EXECUTE_APACHE_SHELL ); }
	public void executeShellNcat() { doIt( EXECUTE_SHELL_NCAT ); }
	public void connectNcatRemote() { doIt( CONNECT_NCAT_REMOTEHOST ); }
	public void executeNcatWget() { doIt( EXECUTE_NCAT_WGET ); }
	public void executeShellWget() { doIt( EXECUTE_SHELL_WGET ); }
	public void executeNcatcc() { doIt( EXECUTE_NCAT_CC ); }
	public void executeShellcc() { doIt( EXECUTE_SHELL_CC ); }
	public void backtoNcat() { doIt( BACKNCAT ); }
	public void backtoShell() { doIt( BACKSHELL ); }
	public void executeNcatExploit() { doIt( EXECUTE_NCAT_EXPLOIT ); }
	public void executeExploitShell() { doIt( EXECUTE_EXPLOIT_SHELL ); }
	public void executeShellUser() { doIt( EXECUTE_SHELL_ADDUSER ); }

}